@@ -14,6 +14,7 @@ import sys
 import platform
 import warnings
 from contextlib import contextmanager
+import subprocess
 
 import numpy as np
 from numpy import (
@@ -35,11 +36,12 @@ import unittest
 TestCase = unittest.TestCase
 
 double = np.double
-
+if sys.version_info[0] >= 3:
+    long = int
 
 # Recommended minimum versions
-minimum_numpy_version = "1.6"
-
+from distutils.version import LooseVersion
+minimum_numpy_version = LooseVersion('1.7.0')
 
 class test_numexpr(TestCase):
     """Testing with 1 thread"""
@@ -295,6 +297,46 @@ class test_numexpr(TestCase):
         b = b'a' * 40
         res = evaluate('contains(a, b)')
         assert_equal(res, True)
+        
+    def test_where_scalar_bool(self):
+        a = True
+        b = array([1, 2])
+        c = array([3, 4])
+        res = evaluate('where(a, b, c)')
+        assert_array_equal(res, b)
+        a = False
+        res = evaluate('where(a, b, c)')
+        assert_array_equal(res, c)
+
+    
+    def test_refcount(self):
+        # Regression test for issue #310
+        a = array([1])
+        assert sys.getrefcount(a) == 2
+        evaluate('1')
+        assert sys.getrefcount(a) == 2
+
+    def test_locals_clears_globals(self):
+        # Check for issue #313, whereby clearing f_locals also clear f_globals
+        # if in the top-frame. This cannot be done inside `unittest` as it is always 
+        # executing code in a child frame.
+        script = r';'.join([
+                r"import numexpr as ne",
+                r"a=10",
+                r"ne.evaluate('1')",
+                r"a += 1",
+                r"ne.evaluate('2', local_dict={})",
+                r"a += 1",
+                r"ne.evaluate('3', global_dict={})",
+                r"a += 1",
+                r"ne.evaluate('4', local_dict={}, global_dict={})",
+                r"a += 1",
+            ])
+        # Raises CalledProcessError on a non-normal exit
+        check = subprocess.check_call([sys.executable, '-c', script])
+        # Ideally this test should also be done against ipython but it's not 
+        # a requirement.
+
 
 
 class test_numexpr2(test_numexpr):
@@ -625,8 +667,7 @@ def test_expressions():
                                                 neval, type(neval), shape(neval))
 
         method.description = ('test_expressions(%s, test_scalar=%r, '
-                              'dtype=%r, optimization=%r, exact=%r)') \
-                             % (expr, test_scalar, dtype.__name__, optimization, exact)
+                              'dtype=%r, optimization=%r, exact=%r)') % (expr, test_scalar, dtype.__name__, optimization, exact)
         test_no[0] += 1
         method.__name__ = 'test_scalar%d_%s_%s_%s_%04d' % (test_scalar,
                                                            dtype.__name__,
@@ -915,17 +956,39 @@ def _environment(key, value):
         else:
             del os.environ[key]
 
-
 # Test cases for the threading configuration
 class test_threading_config(TestCase):
+    def test_max_threads_unset(self):
+        # Has to be done in a subprocess as `importlib.reload` doesn't let us 
+        # re-initialize the threadpool
+        script = '\n'.join([
+                "import os",
+                "if 'NUMEXPR_MAX_THREADS' in os.environ: os.environ.pop('NUMEXPR_MAX_THREADS')",
+                "import numexpr",
+                "assert(numexpr.nthreads <= 8)",
+                "exit(0)"])
+        subprocess.check_call(['python', '-c', script])
+
+    def test_max_threads_set(self):
+        # Has to be done in a subprocess as `importlib.reload` doesn't let us 
+        # re-initialize the threadpool
+        script = '\n'.join([
+                "import os",
+                "os.environ['NUMEXPR_MAX_THREADS'] = '4'",
+                "import numexpr",
+                "assert(numexpr.MAX_THREADS == 4)",
+                "exit(0)"])
+        subprocess.check_call(['python', '-c', script])
+
     def test_numexpr_num_threads(self):
         with _environment('OMP_NUM_THREADS', '5'):
+            # NUMEXPR_NUM_THREADS has priority
             with _environment('NUMEXPR_NUM_THREADS', '3'):
-                self.assertEqual(3, numexpr.detect_number_of_threads())
+                self.assertEqual(3, numexpr._init_num_threads())
 
     def test_omp_num_threads(self):
         with _environment('OMP_NUM_THREADS', '5'):
-            self.assertEqual(5, numexpr.detect_number_of_threads())
+            self.assertEqual(5, numexpr._init_num_threads())
 
 
 # Case test for threads
@@ -995,23 +1058,35 @@ class test_subprocess(TestCase):
 
 def print_versions():
     """Print the versions of software that numexpr relies on."""
-    from pkg_resources import parse_version
-    if parse_version(np.__version__) < parse_version(minimum_numpy_version):
-        print("*Warning*: NumPy version is lower than recommended: %s < %s" % \
-              (np.__version__, minimum_numpy_version))
+    # from pkg_resources import parse_version
+    from numexpr.cpuinfo import cpu
+    import platform
+
+    np_version = LooseVersion(np.__version__)
+
+    if np_version < minimum_numpy_version:
+        print('*Warning*: NumPy version is lower than recommended: %s < %s' % (np_version, minimum_numpy_version))
     print('-=' * 38)
-    print("Numexpr version:   %s" % numexpr.__version__)
-    print("NumPy version:     %s" % np.__version__)
+    print('Numexpr version:   %s' % numexpr.__version__)
+    print('NumPy version:     %s' % np.__version__)
     print('Python version:    %s' % sys.version)
-    if os.name == 'posix':
-        (sysname, nodename, release, version, machine) = os.uname()
-        print('Platform:          %s-%s' % (sys.platform, machine))
-    print("AMD/Intel CPU?     %s" % numexpr.is_cpu_amd_intel)
-    print("VML available?     %s" % use_vml)
+    (sysname, nodename, release, os_version, machine, processor) = platform.uname()
+    print('Platform:          %s-%s-%s' % (sys.platform, machine, os_version))
+    try:
+        # cpuinfo doesn't work on OSX well it seems, so protect these outputs 
+        # with a try block
+        cpu_info = cpu.info[0]
+        print('CPU vendor:        %s' % cpu_info.get('VendorIdentifier', ''))
+        print('CPU model:         %s' % cpu_info.get('ProcessorNameString', ''))
+        print('CPU clock speed:   %s MHz' % cpu_info.get('~MHz',''))
+    except KeyError:
+        pass
+    print('VML available?     %s' % use_vml)
     if use_vml:
-        print("VML/MKL version:   %s" % numexpr.get_vml_version())
-    print("Number of threads used by default: %d "
-          "(out of %d detected cores)" % (numexpr.nthreads, numexpr.ncores))
+        print('VML/MKL version:   %s' % numexpr.get_vml_version())
+    print('Number of threads used by default: %d '
+          '(out of %d detected cores)' % (numexpr.nthreads, numexpr.ncores))
+    print('Maximum number of threads: %s' % numexpr.MAX_THREADS)
     print('-=' * 38)
 
 
